"""
  >>> grok.testing.grok(__name__)

For menu configuration, we have to start a new interaction:

  >>> from Products.Five.security import newInteraction
  >>> newInteraction()

  >>> from zope.component import getUtility, getUtilitiesFor
  >>> from zope.app.publisher.interfaces.browser import IBrowserMenu
  >>> from zope.publisher.browser import TestRequest

A menu is available as a named utility providing ``IBrowserMenu``.

  >>> menu = getUtility(IBrowserMenu, u'tabs')
  >>> manfred = Mammoth()
  >>> request = TestRequest()

In order to retrieve the menu items, we need to pass in a context
object and a request.  The menu then determines which menu items are
available for this particular object and the principal that's attached
to the request:

  >>> from pprint import pprint
  >>> pprint(menu.getMenuItems(manfred, request))
  [{'action': 'edit',
    'description': u'',
    'extra': None,
    'icon': None,
    'selected': u'',
    'submenu': None,
    'title': 'Edit'},
   {'action': 'index',
    'description': u'',
    'extra': None,
    'icon': None,
    'selected': u'',
    'submenu': None,
    'title': 'View'}]

"""

from grokcore.component import Context
from five import grok
from five.grok import testing
from five.megrok import menu
import grokcore.security

class Mammoth(Context):
    pass

class Tabs(menu.Menu):
    grok.name('tabs')
    grok.title('Tabs')
    grok.description('')

# You can either refer to the menu class itself:
# do not forget the security declaration

class Index(grok.View):
    grok.title('View')
    menu.menuitem(Tabs)
    def render(self):
        return 'index'

# or you can refer to its identifier:

class Edit(grok.View):
    grok.title('Edit')
    menu.menuitem('tabs')
    grokcore.security.require('zope2.Public')

    def render(self):
        return 'edit'



# -*- coding: utf-8 -*-
"""
five.megrok.menu

Licensed under the GPL license, see LICENCE.txt for more details.

$Id: event.py 67630 2006-04-27 00:54:03Z jfroche $
"""
import martian
from martian.error import GrokError

import grokcore.component
import grokcore.view
from zope.security.checker import CheckerPublic
from grokcore.view.meta.views import ViewSecurityGrokker, default_view_name

from zope.configuration.exceptions import ConfigurationError
from zope.app.publisher.browser.menumeta import menuDirective, \
    menuItemDirective, subMenuItemDirective

from zope.publisher.interfaces.browser import IDefaultBrowserLayer

import five.megrok.menu


class MenuGrokker(martian.ClassGrokker):
    martian.component(five.megrok.menu.Menu)
    martian.priority(1500)
    martian.directive(grokcore.component.name, get_default=default_view_name)
    martian.directive(grokcore.component.title, default=u'')
    martian.directive(grokcore.component.description, default=u'')

    def execute(self, factory, config, name, title, description, **kw):
        menuDirective(config, id=name, class_=factory,
                      title=title, description=description)
        return True


class SubMenuItemGrokker(martian.ClassGrokker):
    martian.component(five.megrok.menu.SubMenuItem)

    # We want to do this after MenuGrokker.
    martian.priority(1000)

    martian.directive(grokcore.component.name, get_default=default_view_name)
    martian.directive(grokcore.component.title, default=u'')
    martian.directive(grokcore.component.description, default=u'')
    martian.directive(grokcore.component.context)
    martian.directive(grokcore.view.layer, default=IDefaultBrowserLayer)
    martian.directive(grokcore.security.require, name='permission')

    martian.directive(five.megrok.menu.menuitem)

    def execute(self, factory, config, name, title, description, \
                    menuitem=None, context=None, layer=None, permission=None):
        menuDirective(config, id=name, class_=factory,
                      title=title, description=description)

        if menuitem is None:
            return False

        menu_id, icon, filter, order, extra = menuitem
        try:
            menu = config.resolve('zope.app.menus.'+menu_id)
        except ConfigurationError, v:
            raise GrokError("The %r menu could not be found.  Please use "
                            "megrok.menu.Menu to register a menu first."
                            % menu_id, factory)
        if permission is None:
            permission = CheckerPublic

        subMenuItemDirective(config, menu=menu, for_=context, submenu=name,
                          title=title, description=description, icon=icon,
                          filter=filter, permission=permission, layer=layer,
                          order=order, action='', extra=extra)

        return True


class MenuItemGrokker(ViewSecurityGrokker):
    martian.directive(five.megrok.menu.menuitem)
    martian.directive(grokcore.component.context)
    martian.directive(grokcore.view.layer, default=IDefaultBrowserLayer)
    martian.directive(grokcore.component.name, get_default=default_view_name)
    martian.directive(grokcore.component.title, default=u'')
    martian.directive(grokcore.component.description, default=u'')

    def execute(self, factory, config, permission, context=None,
                layer=None, name=u'', menuitem=None, description=u'',
                title=u''):
        if menuitem is None:
            return False
        menu_id, icon, filter, order, extra = menuitem

        try:
            menu = config.resolve('zope.app.menus.'+menu_id)
        except ConfigurationError, v:
            raise GrokError("The %r menu could not be found.  Please use "
                            "megrok.menu.Menu to register a menu first."
                            % menu_id, factory)
        if permission == None:
            permission = CheckerPublic

        menuItemDirective(config, menu=menu, for_=context, action=name,
                          title=title, description=description, icon=icon,
                          filter=filter, permission=permission, layer=layer,
                          order=order, extra=extra)

        return True



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
import os

version = '0.1'

setup(name='five.megrok.menu',
      version=version,
      description="Grok support for browser menu in zope 2",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "License :: OSI Approved :: Zope Public License",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Zope2"],
      keywords='',
      author='Jean-François Roche',
      author_email='jfroche@affinitic.be',
      url='http://svn.zope.org/five.megrok.menu/trunk',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['five', 'five.megrok'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'five.grok'],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



# -*- coding: utf-8 -*-
"""
five.megrok.menu

Licensed under the ZPL license, see LICENCE.txt for more details.

$Id: event.py 67630 2006-04-27 00:54:03Z jfroche $
"""
import martian.util
import grokcore.component
from martian.error import GrokImportError
from zope.app.publisher.browser.menu import BrowserMenu


class Menu(BrowserMenu):
    pass


class SubMenuItem(BrowserMenu):
    pass


class menuitem(martian.Directive):
    scope = martian.CLASS
    store = martian.ONCE

    def factory(self, menu, icon=None, filter=None, order=0, extra=None):
        if martian.util.check_subclass(menu, Menu):
            menu = grokcore.component.name.bind().get(menu)
        if martian.util.not_unicode_or_ascii(menu):
            raise GrokImportError(
                "You can only pass unicode, ASCII, or a subclass "
                "of megrok.menu.Menu to the '%s' directive." % self.name)
        return (menu, icon, filter, order, extra)



import unittest
from pkg_resources import resource_listdir

from zope.testing import doctest
from zope.component import testing

import Products.Five
from Products.Five import zcml
import five.megrok.menu


def setUp(test=None):
    testing.setUp(test)
    zcml.load_config('meta.zcml', package=Products.Five)
    zcml.load_config('configure.zcml', package=Products.Five)
    zcml.load_config('testing.zcml', package=five.megrok.menu)


def suiteFromPackage(name):
    files = resource_listdir(__name__, name)
    suite = unittest.TestSuite()
    for filename in files:
        if not filename.endswith('.py'):
            continue
        if filename.endswith('_fixture.py'):
            continue
        if filename == '__init__.py':
            continue

        dottedname = 'five.megrok.menu.tests.%s.%s' % (name, filename[:-3])
        test = doctest.DocTestSuite(dottedname,
                                    setUp=setUp,
                                    tearDown=testing.tearDown,
                                    optionflags=doctest.ELLIPSIS+
                                                doctest.NORMALIZE_WHITESPACE)

        suite.addTest(test)
    return suite


def test_suite():
    suite = unittest.TestSuite()
    for name in ['menus']:
        suite.addTest(suiteFromPackage(name))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')
